
#include "fccdb.h"
#include "fcc_contour.h"
#include <cassert>
#include <cstring>

void do_station(Facility& fac, Fm_Eng_Data& ed, int count)
{
      std::string callsign = (fac.fac_callsign == "NEW" || fac.fac_callsign == "NONE") 
	? fac.facility_id 
	: fac.fac_callsign;
      std::string out_file_name = ed.station_channel + '-' + callsign 
	+ '-' + char(count + '`') + '-' + ed.fm_dom_status + ".contour";
      std::cout << out_file_name << '\n';
      std::ofstream out(out_file_name.c_str());
      
      double max_power = 
	std::max(std::max(to_double(ed.horiz_erp), to_double(ed.vert_erp)),
		 std::max(to_double(ed.max_horiz_erp), to_double(ed.max_vert_erp)));
      double haat =
	std::max(to_double(ed.max_haat),
		 std::max(to_double(ed.haat_horiz_rc_mtr), to_double(ed.haat_vert_rc_mtr)));

      double rcamsl =
	std::max(to_double(ed.rcamsl_horiz_mtr), to_double(ed.rcamsl_vert_mtr));
      
      out << "#" << fac.fac_callsign << ' ' << fac.comm_city << "," << fac.comm_state << ' '
	  << ed.fm_dom_status << ' ' << fac.fac_status << ' '
	  << fac.facility_id << ' ' << ed.antenna_id << ' ' << ed.last_change_date  << '\n'
	  << ed.lat_deg << ' ' << ed.lat_min << ' ' << ed.lat_sec << ' ' << ed.lat_dir << ' '
	  << ed.lon_deg << ' ' << ed.lon_min << ' ' << ed.lon_sec << ' ' << ed.lon_dir << ' '
	  << max_power << ' ' << ed.station_channel << ' ' << rcamsl << '\n'
	  << "bearing\thaat\tfield\tdb\tpower\tp60\ti40\ti54\ti100\n";
      
      if (ed.antenna_type != "N" && ed.antenna_id != "") {
	for (int pass=(ed.ant_rotation == ""); pass <= 1; ++pass) {
	  std::ifstream f3("ant_pattern.dat");
	  while (f3) {
	    Ant_Pattern ap(f3);
	    if (ed.antenna_id == ap.antenna_id) {
	      double azimuth = to_double(ap.azimuth) + to_double(ed.ant_rotation) ;
	      if ((pass == 0 && azimuth >= 360) || (pass == 1 && azimuth < 360)) {
		if (azimuth >= 360) {
		  azimuth -= 360;
		}else{
		}
		double field = to_double(ap.field_value);
		double db = 20*log10(field);
		double power = max_power * pow(10., db/10.);
		std::ios_base::fmtflags save_flags = out.flags();
		out.setf(std::ios_base::fixed);
		out.precision(0);
		out << azimuth;
		out.precision(1);
		out << '\t' << haat;
		out.precision(3);
		out << '\t' << field;
		out.precision(2);
		out << '\t' << db;
		out.flags(save_flags);
		out.precision(3);
		out << '\t' << power;
		out.precision(4);
		out << '\t'
		    << contour_distance(power, haat, 60., PROTECTED, 211) << '\t'
		    << contour_distance(power, haat, 40., INTERFERING, 211) << '\t'
		    << contour_distance(power, haat, 54., INTERFERING, 211) << '\t'
		    << contour_distance(power, haat, 100., INTERFERING, 211) << '\n';
	      }else{
		// wrong pass
	      }
	    }else{
	      // wrong antenna
	    }
	  }
	}	  
      }else{
	// non-directional antenna
	double power = max_power;
	for (double azimuth = 0.; azimuth < 360.; azimuth += 10.) {
	  std::ios_base::fmtflags save_flags = out.flags();
	  out.setf(std::ios_base::fixed);
	  out.precision(0);
	  out << azimuth;
	  out.precision(1);
	  out << '\t' << haat
	      << "\t1.0\t0.0";
	  out.flags(save_flags);
	  out.precision(3);
	  out << '\t' << power;
	  out.precision(4);
	  out << '\t'
	      << contour_distance(power, haat, 60., PROTECTED, 211) << '\t'
	      << contour_distance(power, haat, 40., INTERFERING, 211) << '\t'
	      << contour_distance(power, haat, 54., INTERFERING, 211) << '\t'
	      << contour_distance(power, haat, 100., INTERFERING, 211) << '\n';
	}
      }
      out << '\n';

}


int main(int argc, char* argv[])
{
  const int LIST_LENGTH = 20;
  {
    if (argc == 1) {
      // no args -- print a record of everything
      std::ifstream f1("facility.dat");
      while (f1) {
	Facility d(f1);
	d.dump();
      }
    }else if (argv[1][0] == '-') {
      // a group of records, selected by lat, long, channel, state
      std::string latitude[LIST_LENGTH], longitude[LIST_LENGTH],
	channel[LIST_LENGTH], state[LIST_LENGTH], service[LIST_LENGTH], status[LIST_LENGTH];
      int i = 1;
      while (i < argc) {
	if (strncmp(argv[i], "-la", 3) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    latitude[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-lo", 3) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    longitude[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-c", 2) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    channel[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-status", 7) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    status[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-service", 8) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    service[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-s", 2) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < LIST_LENGTH) {
	    state[j++] = argv[i++];
	  }
	}else{
	  std::cout << "bad arg: " << argv[i] << '\n';
	}
      }
      std::ifstream f1("facility.dat");
      while (f1) {
	Facility fac(f1);
	bool service_matches  = ((service[0] == "" && fac.fac_service == "FM")
				 || service[0] == fac.fac_service);
	for (int i = 1; !service_matches && i < LIST_LENGTH && service[i] != ""; ++i) {
	  service_matches = (service[i] == fac.fac_service);
	}
	bool status_matches  = (status[0] == "" || status[0] == fac.fac_status);
	for (int i = 1; !status_matches && i < LIST_LENGTH && status[i] != ""; ++i) {
	  status_matches = (status[i] == fac.fac_status);
	}
	bool state_matches  = (state[0] == "" || state[0] == fac.comm_state);
	for (int i = 1; !state_matches && i < LIST_LENGTH && state[i] != ""; ++i) {
	  state_matches = (state[i] == fac.comm_state);
	}
	if (service_matches && status_matches && state_matches) {
	  std::cout << fac.fac_callsign << '\n';
	  std::ifstream f2("fm_eng_data.dat");
	  int count = 0;
	  while (f2) {
	    Fm_Eng_Data ed(f2);
	    if (fac.facility_id == ed.facility_id) {
	      ++count;
	      bool channel_matches = (channel[0] == "" || channel[0] == ed.station_channel);
	      for (int i = 1; !channel_matches && i < LIST_LENGTH && channel[i] != ""; ++i) {
		channel_matches = (channel[i] == ed.station_channel);
	      }
	      bool latitude_matches = (latitude[0] == "" || latitude[0] == ed.lat_deg);
	      for (int i = 1; !latitude_matches && i < LIST_LENGTH && latitude[i] != ""; ++i) {
		latitude_matches = (latitude[i] == ed.lat_deg);
	      }
	      bool longitude_matches = (longitude[0] == "" || longitude[0] == ed.lon_deg);
	      for (int i = 1; !longitude_matches && i < LIST_LENGTH && longitude[i] != ""; ++i) {
		longitude_matches = (longitude[i] == ed.lon_deg);
	      }
	      if (channel_matches && latitude_matches && longitude_matches) {
		do_station(fac, ed, count);
	      }else{
	      }
	    }else{
	    }
	  }
	}
      }
    }else{
      // one station .. argv[1] is either the facility id or callsign
      std::ifstream f1("facility.dat");
      while (f1) {
	Facility fac(f1);
	if ((argv[1] == fac.facility_id) 
	    || (fac.fac_callsign.compare(0, strlen(argv[1]), argv[1]) == 0)) {
	  std::ifstream f2("fm_eng_data.dat");
	  int count = 0;
	  while (f2) {
	    Fm_Eng_Data ed(f2);
	    if (fac.facility_id == ed.facility_id) {
	      ++count;
	      do_station(fac, ed, count);
	    }else{
	    }
	  }
	}else{
	}
      }
    }
  }
  return 0;
}
